#include <Redis_Provider.h>
#include <sstream>
#include <regex>

#include <ctime>
#include <iomanip>


using namespace inaf::oasbo::Providers;
RedisProvider::RedisProvider() :
		RedisProvider("127.0.0.1", 6379, "Astri_ADAS_key") {
}

RedisProvider::RedisProvider(std::string ip, int port, std::string key) :
		ip(ip), port(port), key(key) {
}

int RedisProvider::write(PacketLib::BasePacket &packet) {
	return write(packet, this->key);
}

int RedisProvider::write(PacketLib::BasePacket &packet, std::string key) {
	if (!isOpen()) {
		if (context) {
			time_t now = time(nullptr);
			std::cerr << "["
					<< std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
					<< "]\t[Redis Provider]\t" << context->errstr << std::endl;
		} else {
			time_t now = time(nullptr);
			std::cerr << "["
					<< std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
					<< "]\t[Redis Provider]\t" << "Can't allocate Redis context"
					<< std::endl;
		}
		return -1;
	}

	uint size = packet.getHeaderSize() + packet.getPayloadSize()
			+ packet.getTailSize();
	redisReply *r = (redisReply*) redisCommand(context, "LPUSH %s %b",
			key.c_str(), (char*) packet.getPointerToMemory(), size);
	if (r == NULL) {
		// Error executing Redis command
		return -1;
	}
	// Check the type of the Redis reply
	if (r->type == REDIS_REPLY_ERROR) {
		// Error executing Redis command
		time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[Redis Provider]\t" << r->str << std::endl;
		freeReplyObject(r);
		return -1;
	}
	freeReplyObject(r);
	return size;
}

int RedisProvider::open() {
	if (isOpen())
		close();
	context = redisConnect(ip.c_str(), port);
	if (context == nullptr || context->err) {
		if (context) {
			time_t now = time(nullptr);
			std::cerr << "["
					<< std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
					<< "]\t[Redis Provider]\t" << context->errstr << std::endl;
		} else {
			time_t now = time(nullptr);
			std::cerr << "["
					<< std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
					<< "]\t[Redis Provider]\t" << "Can't allocate Redis context"
					<< std::endl;
		}
		return -1;
	}
	// flush queue
	redisReply *r = (redisReply*) redisCommand(context, "DEL %s",
			this->key.c_str());
	if (r == NULL) {
		time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[Redis Provider]\t"
				<< "error in flush queue with cmd: DEL " << this->key.c_str()
				<< std::endl;
		return -1;
	}
	if (r->type == REDIS_REPLY_ERROR) {
		// Error executing Redis command
		time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[Redis Provider]\t"
				<< "error in flush queue with cmd: DEL " << this->key.c_str()
				<< std::endl;
		freeReplyObject(r);
		return -1;
	}

	return 1;
}

int RedisProvider::close() {
	if (context != nullptr && !context->err) {
		redisFree(this->context);
		context = nullptr;
	}
	return 1;
}

bool RedisProvider::isOpen() {
	return (context != nullptr && !context->err);
}

void RedisProvider::setDest(std::string dest) {
	std::regex pattern(
			R"((\d+\.\d+\.\d+\.\d+):(\d+)/(.+))");
    std::smatch matches;
    if (std::regex_match(dest, matches, pattern) && matches.size() == 4) {
         ip = matches[1].str();
         port = std::stoi(matches[2].str());
         key = matches[3].str();
    }
	std::stringstream ss;
	ss << ip << ":" << port << "/" << key;
	this->dest = ss.str();

}
std::string RedisProvider::getDest(){
	std::stringstream ss;
	ss << ip << ":" << port << "/" << key;
	return ss.str();
}
RedisProvider::~RedisProvider() {
	close();
}

